DECLARE FUNCTION Dir$ (filespec$, attrib AS INTEGER)
'
'CALL INTERRUPT - a statement that allows BASIC programs to perform
'                 DOS system calls
'Syntax
'  CALL INTERRUPT(interruptnum,inregs,outregs)
'     interruptnum, an integer expression which has a value between
'      0 and 255, is a DOS interrupt number
'     inregs contains the register values before the interrupt is performed
'      (see Details for how to declare a user-defined type which will
'      contain these register values)
'     outregs contains the register values after the interrupt

'Syntax Detail
'  CALL INTERRUPT  (interruptnum, inregs, outregs)
'
'  Argument       Description
'  interruptnum   The DOS interrupt number. It is an integer between
'                 0 and 255. See your DOS documentation for informa-
'                 tion about interrupts.
'  inregs         The inregs variable contains the register values
'                 used when the interrupt is performed. It is declared
'                 as type RegType. The user-defined type RegType is
'                 described below.
'  outregs        The outregs variable contains the register values
'                 after the interrupt is performed. It is declared
'                 as type RegType. The user-defined type RegType is
'                 described below.
'
'The CALL INTERRUPT statement replaces the INT86 and INT86X routines
'used in earlier versions of BASIC. It provides a more convenient way
'for BASIC programs to use DOS interrupts and services.
'
'The register values before and after the interrupt are passed in
'variables declared as type RegType. The following statement defines
'the RegType user-defined type (see below)
'
'Each element of the type corresponds to a CPU register.
'
'NOTE for QuickBasic users: You cannot use code like the following:
'
'  regs.ds = -1         'use default (QuickBasic only!!)
'  regs.si = VARPTR(v)
'
'Instead, you must do something like this:
'
'  regs.ds = VARSEG(v)
'  regs.si = VARPTR(v)  'Then it will have the right offset
'
'SUB Dir$() in this file has an example of this
TYPE RegType
    ax    AS INTEGER
    bx    AS INTEGER
    cx    AS INTEGER
    dx    AS INTEGER
    bp    AS INTEGER
    si    AS INTEGER
    di    AS INTEGER
    flags AS INTEGER
    ds    AS INTEGER
    es    AS INTEGER
END TYPE
DECLARE SUB INTERRUPT (interruptnum AS INTEGER, inregs AS RegType, outregs AS RegType)

'example (see SUB Dir$()): getting a list of files in the current directory
f$ = Dir$("*.*", 0)
WHILE f$ <> ""
    PRINT f$,
    f$ = Dir$("", 0)
WEND

DEFINT A-Z
'***************************************************************************
'** Dir$: Searches for the first/next occurrence of a filename
'**
'**  filespec$: Filename to match. Wildcards may be used. If the next occur-
'**             ence of the filename is wanted, pass "" for filespec.
'**  attrib: Attributes to match; a combination of the following:
'**             1 - Read Only
'**             2 - Hidden
'**             4 - System
'**             8 - Volume Label
'**             16 - Directory
'**             32 - Archive
'**
'** Returns a filename, or "" if no more files found
'***************************************************************************
FUNCTION Dir$ (filespec$, attrib AS INTEGER) STATIC

    CONST DOS = &H21
    CONST SetDTA = &H1A00, FindFirst = &H4E00, FindNext = &H4F00
 
    DIM DTA AS STRING * 128, regs AS RegType
    Null$ = CHR$(0)

ReTry:

    '-----  Set up our own DTA so we don't destroy COMMAND$
    regs.ax = SetDTA                    'Set DTA function
    DEF SEG = VARSEG(DTA)
    regs.dx = VARPTR(DTA)               'DS:DX points to our DTA
    regs.ds = VARSEG(DTA)               'Use current value for DS
    DEF SEG
    INTERRUPT DOS, regs, regs          'Do the interrupt

    '-----  Check to see if this is First or Next
    IF filespec$ <> "" THEN            'FileSpec$ isn't null, so FindFirst
        FileSpecZ$ = filespec$ + Null$   'Make FileSpec$ into an ASCIIZ
                                            'string
        regs.ax = FindFirst              'Perform a FindFirst
        regs.cx = attrib
        DEF SEG = VARSEG(FileSpecZ$)
        regs.dx = SADD(FileSpecZ$)       'DS:DX points to ASCIIZ file
        regs.ds = VARSEG(FileSpecZ$)     'Use current DS
        DEF SEG
    ELSE                                 'We have a null FileSpec$,
        regs.ax = FindNext               'so FindNext
    END IF

    INTERRUPT DOS, regs, regs          'Do the interrupt

     '-----  Return file name or null
     IF regs.flags AND 1 THEN            'No files found
         Dir$ = ""                        'Return null string
         EXIT FUNCTION
     ELSE
        '* check if its . or .., if so, can't accept it.
        IF ASC(MID$(DTA, 22, 1)) <> attrib AND attrib <> 0 THEN
            'attribs don't match, so skip it
             filespec$ = ""
             GOTO ReTry
        END IF
        IF MID$(DTA, 31, 1) = "." THEN   '*If its . or ..
            filespec$ = ""
            GOTO ReTry
        END IF
     END IF
     Null = INSTR(31, DTA, Null$)     'Get the filename found
     Dir$ = MID$(DTA, 31, Null - 31)  'It's an ASCIIZ string starting
                                         'at offset 30 of the DTA
END FUNCTION

DEFSNG A-Z
SUB INTERRUPT (interruptnum AS INTEGER, inregs AS RegType, outregs AS RegType)
    STATIC intcode() AS INTEGER
    STATIC madecode AS INTEGER
    DIM t AS LONG, o AS LONG
 
    'Initialize assembly code once only
    IF madecode = 0 THEN
        REDIM intcode(0 TO 73) AS INTEGER
       
        'Initialize assembly language code
             h$ = "EB140000000000000000000000000000"
        h$ = h$ + "000000000000505351525556571E069C"
        h$ = h$ + "2EA10200502EA104002E8B1E06002E8B"
        h$ = h$ + "0E08002E8B160A002E8B2E0C002E8B36"
        h$ = h$ + "0E002E8B3E10002E8E1E12002E8E0614"
        h$ = h$ + "009DCD16509C582EA30200582EA30400"
        h$ = h$ + "2E891E06002E890E08002E89160A002E"
        h$ = h$ + "892E0C002E89360E002E893E10002E8C"
        h$ = h$ + "1E12002E8C0614009D071F5F5E5D5A59"
        h$ = h$ + "5B58CB00"

        'Place it in the array
        DEF SEG = VARSEG(intcode(0))
        FOR i = 0 TO &H92
            t = VAL("&H" + MID$(h$, i * 2 + 1, 2))
            POKE VARPTR(intcode(0)) + i, t
        NEXT
        DEF SEG
        madecode = 1
    END IF

    'Using -1 to use default DS (QuickBasic style) won't work here, so
    'make an error if someone tries to do it.
    IF inregs.ds = -1 THEN ERROR 5

    'Load the input
    intcode(1) = inregs.flags
    intcode(2) = inregs.ax
    intcode(3) = inregs.bx
    intcode(4) = inregs.cx
    intcode(5) = inregs.dx
    intcode(6) = inregs.bp
    intcode(7) = inregs.si
    intcode(8) = inregs.di
    intcode(9) = inregs.ds
    intcode(10) = inregs.es
   
    t = &HCD OR (interruptnum AND &HFF) * 256&
    IF t > 32767& THEN t = t - 65536
    intcode(41) = t

    'Call the routine
    DEF SEG = VARSEG(intcode(0))
    CALL ABSOLUTE(VARPTR(intcode(0)))
    DEF SEG
   
    'Get the output
    inregs.flags = intcode(1)
    inregs.ax = intcode(2)
    inregs.bx = intcode(3)
    inregs.cx = intcode(4)
    inregs.dx = intcode(5)
    inregs.bp = intcode(6)
    inregs.si = intcode(7)
    inregs.di = intcode(8)
    inregs.ds = intcode(9)
    inregs.es = intcode(10)
END SUB

